@@ -1,7 +1,18 @@
 static USAGE: &str = r#"
-Prints a count of the number of records in the CSV data. If the polars feature is
-enabled, it will use the much faster multithreaded, mem-mapped Polars CSV reader.
-Otherwise, it will use the regular, single-threaded CSV reader.
+Returns a count of the number of records in the CSV data.
+
+It has three modes of operation:
+ 1. If a valid index is present, it will use it to lookup the count and
+    return instantaneously. (fastest)
+
+ If no index is present, it will read the CSV and count the number
+ of records by scanning the file. 
+   
+   2. If the polars feature is enabled, it will use the multithreaded,
+      mem-mapped Polars CSV reader. (faster - not available on qsvlite)
+      
+   3. If the polars feature is not enabled, it will use the "regular",
+      single-threaded CSV reader.
 
 Note that the count will not include the header row (unless --no-headers is
 given).
@@ -18,6 +29,9 @@ count options:
                            Its an estimate as it doesn't count quotes, and will be an
                            undercount if the record has quoted fields.
                            The count and width are separated by a semicolon.
+                           Note that this option will require scanning the entire file
+                           using the "regular", single-threaded, streaming CSV reader
+                           and will not use the index nor the Polars CSV reader.
 
                            WHEN THE POLARS FEATURE IS ENABLED:
     --no-polars            Use the "regular", single-threaded, streaming CSV reader instead
@@ -25,8 +39,8 @@ count options:
                            Use this when you encounter memory issues when counting with the
                            Polars CSV reader. The streaming reader is slower but can read
                            any valid CSV file of any size.
-    --low-memory           Use the Polars CSV Reader's low-memory mode. This
-                           mode is slower but uses less memory. If counting still fails,
+    --low-memory           Use the Polars CSV Reader's low-memory mode. This mode
+                           is slower but uses less memory. If counting still fails,
                            use --no-polars instead to use the streaming CSV reader.
 
 
@@ -164,7 +178,7 @@ pub fn polars_count_input(
     conf: &Config,
     low_memory: bool,
 ) -> Result<(u64, usize), crate::clitypes::CliError> {
-    use polars::{prelude::*, sql::SQLContext};
+    use polars::{lazy::frame::LazyFrame, prelude::*, sql::SQLContext};
 
     log::info!("using polars");
 
@@ -195,42 +209,63 @@ pub fn polars_count_input(
         None
     };
 
-    let optimization_state = polars::lazy::frame::OptState {
-        projection_pushdown: true,
-        predicate_pushdown:  true,
-        type_coercion:       true,
-        simplify_expr:       true,
-        file_caching:        true,
-        slice_pushdown:      true,
-        comm_subplan_elim:   false,
-        comm_subexpr_elim:   true,
-        streaming:           true,
-        fast_projection:     true,
-        eager:               false,
-        row_estimate:        true,
+    let mut ctx = SQLContext::new();
+    let lazy_df: LazyFrame;
+    let delimiter = conf.get_delimiter();
+
+    // if its a "regular" CSV, use polars' read_csv() SQL table function
+    // which is much faster than the LazyCsvReader
+    let count_query = if comment_prefix.is_none() && delimiter == b',' && !low_memory {
+        format!(
+            "SELECT COUNT(*) FROM read_csv('{}')",
+            filepath.to_string_lossy(),
+        )
+    } else {
+        // otherwise, read the file into a Polars LazyFrame
+        // using the LazyCsvReader builder to set CSV read options
+        lazy_df = match LazyCsvReader::new(filepath.clone())
+            .with_separator(delimiter)
+            .with_comment_prefix(comment_prefix)
+            .with_low_memory(low_memory)
+            .finish()
+        {
+            Ok(lazy_df) => lazy_df,
+            Err(e) => {
+                log::warn!("polars error loading CSV: {e}");
+                let (count_regular, _) = count_input(conf, false)?;
+                return Ok((count_regular, 0));
+            },
+        };
+        let optimization_state = polars::lazy::frame::OptState {
+            projection_pushdown:  true,
+            predicate_pushdown:   true,
+            cluster_with_columns: true,
+            type_coercion:        true,
+            simplify_expr:        true,
+            file_caching:         true,
+            slice_pushdown:       true,
+            comm_subplan_elim:    false,
+            comm_subexpr_elim:    true,
+            streaming:            true,
+            fast_projection:      true,
+            eager:                false,
+            row_estimate:         true,
+        };
+        ctx.register("sql_lf", lazy_df.with_optimizations(optimization_state));
+        "SELECT COUNT(*) FROM sql_lf".to_string()
     };
 
-    // read the file into a Polars LazyFrame
-    let lazy_df = match LazyCsvReader::new(filepath.clone())
-        .with_separator(conf.get_delimiter())
-        .with_comment_prefix(comment_prefix)
-        .low_memory(low_memory)
-        .finish()
-    {
-        Ok(lazy_df) => lazy_df,
+    // now leverage the magic of Polars SQL with its lazy evaluation, to count the records
+    // in an optimized manner with its blazing fast multithreaded, mem-mapped CSV reader!
+    let sqlresult_lf = match ctx.execute(&count_query) {
+        Ok(sqlresult_lf) => sqlresult_lf,
         Err(e) => {
-            log::warn!("polars error: {}", e);
+            log::warn!("polars error executing count query: {e}");
             let (count_regular, _) = count_input(conf, false)?;
             return Ok((count_regular, 0));
         },
     };
 
-    // and leverage the magic of Polars SQL with its lazy evaluation, to count the records
-    // in an optimized manner with its blazing fast multithreaded, mem-mapped CSV reader!
-    let mut ctx = SQLContext::new();
-    ctx.register("sql_lf", lazy_df.with_optimizations(optimization_state));
-    let sqlresult_lf = ctx.execute("SELECT COUNT(*) FROM sql_lf")?;
-
     let mut count = if let Ok(cnt) = sqlresult_lf.collect()?["len"].u32() {
         cnt.get(0).ok_or("polars error: cannot get count")? as u64
     } else {
