@@ -643,28 +643,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // amortize allocations
     let mut record = csv::StringRecord::with_capacity(500, col_count);
     let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
-    let mut col_name: String;
 
-    // process the first row as the header row
+    // get headers
     info!("exporting sheet ({sheet})... processing first row as header...");
-    let first_row = match rows_iter.next() {
-        Some(first_row) => first_row,
-        None => &[Data::Empty],
-    };
-    for cell in first_row {
-        col_name = match *cell {
-            Data::String(ref s) => s.to_string(),
-            Data::Empty => String::new(),
-            Data::Error(ref _e) => String::new(),
-            Data::Int(ref i) => i.to_string(),
-            Data::Float(ref f) => f.to_string(),
-            Data::DateTime(ref exceldatetime) => exceldatetime.to_string(),
-            Data::Bool(ref b) => b.to_string(),
-            Data::DateTimeIso(ref dt) => dt.to_string(),
-            Data::DurationIso(ref d) => d.to_string(),
-        };
-        record.push_field(&col_name);
+    let headers = range.headers().unwrap_or_default();
+    for header in headers {
+        record.push_field(&header);
     }
+    rows_iter.next(); // we processed the header row
 
     let trim = args.flag_trim;
 
@@ -708,6 +694,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let chunk_size = row_count.div_ceil(ncpus);
 
     let keep_zero_time = args.flag_keep_zero_time;
+    let formula_get_value_error = "cannot get formula".to_string();
 
     let processed_rows: Vec<Vec<csv::StringRecord>> = rows
         .par_chunks(chunk_size)
@@ -723,8 +710,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let mut processed_chunk: Vec<csv::StringRecord> = Vec::with_capacity(chunk_size);
             let mut col_idx = 0_u32;
 
-            let formula_get_value_error = "cannot get formula".to_string();
-            let mut cell_value: &String;
+            let mut cell_formula;
 
             for (row_idx, row) in chunk {
                 for cell in *row {
@@ -800,18 +786,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         Data::DateTimeIso(ref dt) => record.push_field(dt),
                         Data::DurationIso(ref d) => record.push_field(d),
                         Data::Error(ref e) => {
-                            match error_format {
-                                ErrorFormat::Code => record.push_field(&format!("{e}")),
-                                ErrorFormat::Formula | ErrorFormat::Both => {
-                                    cell_value = sheet_formulas
-                                        .get_value((*row_idx, col_idx))
-                                        .unwrap_or(&formula_get_value_error);
-                                    if error_format == ErrorFormat::Formula {
-                                        record.push_field(&format!("#={cell_value}"));
-                                    } else {
-                                        record.push_field(&format!("{e}: ={cell_value}"));
-                                    }
-                                },
+                            if error_format == ErrorFormat::Code {
+                                record.push_field(&format!("{e}"));
+                            } else {
+                                cell_formula = sheet_formulas
+                                    .get_value((*row_idx, col_idx))
+                                    .unwrap_or(&formula_get_value_error);
+                                if error_format == ErrorFormat::Formula {
+                                    record.push_field(&format!("#={cell_formula}"));
+                                } else {
+                                    // ErrorFormat::Both
+                                    record.push_field(&format!("{e}: ={cell_formula}"));
+                                }
                             };
                         },
                     };
