@@ -127,6 +127,8 @@ pub struct Args {
     pub flag_memcheck:       bool,
 }
 
+const NULL_VAL: &[u8] = b"(NULL)";
+
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
     let rconfig = args.rconfig();
@@ -165,7 +167,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let mut sorted_counts: Vec<(Vec<u8>, u64, f64)> = args.counts(&ftab);
 
         // if not --other_sorted and the first value is "Other (", rotate it to the end
-        if !args.flag_other_sorted && sorted_counts.first().unwrap().0.starts_with(b"Other (") {
+        if !args.flag_other_sorted
+            && sorted_counts.first().is_some_and(|(value, _, _)| {
+                value.starts_with(format!("{} (", args.flag_other_text).as_bytes())
+            })
+        {
             sorted_counts.rotate_left(1);
         }
 
@@ -234,13 +240,13 @@ impl Args {
         if self.flag_lmt_threshold == 0 || self.flag_lmt_threshold >= unique_counts_len {
             // check if the column has all unique values
             // by checking if counts length is equal to ftable length
-            let pos_limit = self.flag_limit.unsigned_abs();
+            let abs_limit = self.flag_limit.unsigned_abs();
             let unique_limited = if self.flag_limit > 0
-                && self.flag_unq_limit != pos_limit
+                && self.flag_unq_limit != abs_limit
                 && self.flag_unq_limit > 0
                 && unique_counts_len == ftab.len()
             {
-                counts = counts.into_iter().take(self.flag_unq_limit).collect();
+                counts.truncate(self.flag_unq_limit);
                 true
             } else {
                 false
@@ -248,31 +254,36 @@ impl Args {
 
             // check if we need to limit the number of values
             if self.flag_limit > 0 {
-                counts = counts.into_iter().take(pos_limit).collect();
+                counts.truncate(abs_limit);
             } else if self.flag_limit < 0 && !unique_limited {
                 // if limit is negative, only return values with an occurence count >= absolute
                 // value of the negative limit. We only do this if we haven't
                 // already unique limited the values
-                let count_limit = pos_limit as u64;
-                counts.retain(|(_, c)| *c >= count_limit);
+                let count_limit = abs_limit as u64;
+                counts.retain(|(_, count)| *count >= count_limit);
             }
         }
 
-        const NULL_VAL: &[u8] = b"(NULL)";
         let mut pct_sum = 0.0_f64;
         let mut pct = 0.0_f64;
         let mut count_sum = 0_u64;
+        let pct_factor = if total_count > 0 {
+            100.0_f64 / total_count.to_f64().unwrap_or(1.0_f64)
+        } else {
+            0.0_f64
+        };
 
+        #[allow(clippy::cast_precision_loss)]
         let mut counts_final: Vec<(Vec<u8>, u64, f64)> = counts
             .into_iter()
-            .map(|(bs, c)| {
-                count_sum += c;
-                pct = (c as f64 * 100.0_f64) / total_count as f64;
+            .map(|(byte_string, count)| {
+                count_sum += count;
+                pct = count as f64 * pct_factor;
                 pct_sum += pct;
-                if b"" == &**bs {
-                    (NULL_VAL.to_vec(), c, pct)
+                if *b"" == **byte_string {
+                    (NULL_VAL.to_vec(), count, pct)
                 } else {
-                    (bs.clone(), c, pct)
+                    (byte_string.to_owned(), count, pct)
                 }
             })
             .collect();
