@@ -52,6 +52,10 @@ joinp options:
                            number of rows in the given data sets, respectively.
                            The columns1 and columns2 arguments are ignored.
 
+    --coalesce             Force the join to coalesce columns with the same name.
+                           For inner joins, this is not necessary as the join
+                           columns are automatically coalesced.
+
     --filter-left <arg>    Filter the left CSV data set by the given Polars SQL
                            expression BEFORE the join. Only rows that evaluates
                            to true are used in the join.
@@ -90,6 +94,10 @@ joinp options:
                            will be skipped. If not set, the query will fail.
                            Only use this when debugging queries, as polars does batched
                            parsing and will skip the entire batch where the error occurred.
+    --decimal-comma        Use comma as the decimal separator when parsing CSVs.
+                           Otherwise, use period as the decimal separator.
+                           Note that you'll need to set --delimiter to an alternate delimiter
+                           other than the default comma if you are using this option.
 
                            ASOF JOIN OPTIONS:
     --asof                 Do an 'asof' join. This is similar to a left inner
@@ -183,6 +191,7 @@ use polars::{
     },
     sql::SQLContext,
 };
+use polars_ops::frame::JoinCoalesce;
 use serde::Deserialize;
 use smartstring::SmartString;
 use tempfile::tempdir;
@@ -200,12 +209,14 @@ struct Args {
     flag_left_semi:        bool,
     flag_full:             bool,
     flag_cross:            bool,
+    flag_coalesce:         bool,
     flag_filter_left:      Option<String>,
     flag_filter_right:     Option<String>,
     flag_validate:         Option<String>,
     flag_nulls:            bool,
     flag_streaming:        bool,
     flag_try_parsedates:   bool,
+    flag_decimal_comma:    bool,
     flag_infer_len:        usize,
     flag_low_memory:       bool,
     flag_no_optimizations: bool,
@@ -267,9 +278,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         (true, false, false, false, false, false) => join.run(JoinType::Left, validation, false),
         (false, true, false, false, false, false) => join.run(JoinType::Anti, validation, false),
         (false, false, true, false, false, false) => join.run(JoinType::Semi, validation, false),
-        (false, false, false, true, false, false) => {
-            join.run(JoinType::Outer { coalesce: true }, validation, false)
-        },
+        (false, false, false, true, false, false) => join.run(JoinType::Full, validation, false),
         (false, false, false, false, true, false) => join.run(JoinType::Cross, validation, false),
         (false, false, false, false, false, true) => {
             // safety: flag_strategy is always is_some() as it has a default value
@@ -335,6 +344,7 @@ struct JoinStruct {
     right_sel:        String,
     output:           Option<String>,
     delim:            u8,
+    coalesce:         bool,
     streaming:        bool,
     no_optimizations: bool,
     sql_filter:       Option<String>,
@@ -373,6 +383,12 @@ impl JoinStruct {
             );
         }
 
+        let coalesce_flag = if self.coalesce {
+            JoinCoalesce::CoalesceColumns
+        } else {
+            JoinCoalesce::JoinSpecific
+        };
+
         let optimization_state = if self.no_optimizations {
             // use default optimization state
             polars::lazy::frame::OptState {
@@ -381,18 +397,19 @@ impl JoinStruct {
             }
         } else {
             polars::lazy::frame::OptState {
-                projection_pushdown: true,
-                predicate_pushdown:  true,
-                type_coercion:       true,
-                simplify_expr:       true,
-                file_caching:        true,
-                slice_pushdown:      true,
-                comm_subplan_elim:   true,
-                comm_subexpr_elim:   true,
-                streaming:           self.streaming,
-                fast_projection:     true,
-                eager:               false,
-                row_estimate:        true,
+                projection_pushdown:  true,
+                predicate_pushdown:   true,
+                cluster_with_columns: true,
+                type_coercion:        true,
+                simplify_expr:        true,
+                file_caching:         true,
+                slice_pushdown:       true,
+                comm_subplan_elim:    true,
+                comm_subexpr_elim:    true,
+                streaming:            self.streaming,
+                fast_projection:      true,
+                eager:                false,
+                row_estimate:         true,
             }
         };
         log::debug!("Optimization state: {optimization_state:?}");
@@ -403,6 +420,7 @@ impl JoinStruct {
                 .join_builder()
                 .with(self.right_lf.with_optimizations(optimization_state))
                 .how(JoinType::Cross)
+                .coalesce(coalesce_flag)
                 .allow_parallel(true)
                 .validate(validation)
                 .finish()
@@ -432,6 +450,7 @@ impl JoinStruct {
                 .left_on(left_selcols)
                 .right_on(right_selcols)
                 .how(jointype)
+                .coalesce(coalesce_flag)
                 .allow_parallel(true)
                 .validate(validation)
                 .finish()
@@ -526,13 +545,14 @@ impl Args {
             }
 
             LazyCsvReader::new(&self.arg_input1)
-                .has_header(true)
+                .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
                 .with_separator(tsvtab_delim(&self.arg_input1, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
-                .low_memory(low_memory)
+                .with_decimal_comma(self.flag_decimal_comma)
+                .with_low_memory(low_memory)
                 .with_ignore_errors(ignore_errors)
                 .finish()?
         };
@@ -551,13 +571,14 @@ impl Args {
             }
 
             LazyCsvReader::new(&self.arg_input2)
-                .has_header(true)
+                .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
                 .with_separator(tsvtab_delim(&self.arg_input2, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
-                .low_memory(low_memory)
+                .with_decimal_comma(self.flag_decimal_comma)
+                .with_low_memory(low_memory)
                 .with_ignore_errors(ignore_errors)
                 .finish()?
         };
@@ -574,6 +595,7 @@ impl Args {
             right_sel: self.arg_columns2.clone(),
             output: self.flag_output.clone(),
             delim,
+            coalesce: self.flag_coalesce,
             streaming: self.flag_streaming,
             no_optimizations: self.flag_no_optimizations,
             sql_filter: self.flag_sql_filter.clone(),
