@@ -1,6 +1,6 @@
 static USAGE: &str = r#"
 Create multiple new computed columns, filter rows or compute aggregations by 
-executing a Luau 0.622 script for every row (SEQUENTIAL MODE) or for
+executing a Luau 0.625 script for every row (SEQUENTIAL MODE) or for
 specified rows (RANDOM ACCESS MODE) of a CSV file.
 
 Luau is not just another qsv command. It is qsv's Domain-Specific Language (DSL)
@@ -10,17 +10,22 @@ The executed Luau has 3 ways to reference row columns (as strings):
   1. Directly by using column name (e.g. Amount), can be disabled with -g
   2. Indexing col variable by column name: col.Amount or col["Total Balance"]
   3. Indexing col variable by column 1-based index: col[1], col[2], etc.
-     This is only available with the --colindex and --no-headers options.
+     This is only available with the --colindex or --no-headers options.
 
 Of course, if your input has no headers, then 3. will be the only available
 option.
 
+It has two subcommands:
+  map     - Create new columns by mapping the result of a Luau script for each row.
+  filter  - Filter rows by executing a Luau script for each row. Rows that return
+            true are kept, the rest are filtered out.
+
 Some usage examples:
 
   Sum numeric columns 'a' and 'b' and call new column 'c'
   $ qsv luau map c "a + b"
   $ qsv luau map c "col.a + col['b']"
-  $ qsv luau map c "col[1] + col[2]"
+  $ qsv luau map c --colindex "col[1] + col[2]"
 
   There is some magic in the previous example as 'a' and 'b' are passed in
   as strings (not numbers), but Luau still manages to add them up.
@@ -101,8 +106,8 @@ SPECIAL VARIABLES:
 
 For security and safety reasons as a purpose-built embeddable interpreter,
 Luau's standard library is relatively minimal (https://luau-lang.org/library).
-That's why qsv preloads the LuaDate library as date manipulation is a common task.
-See https://tieske.github.io/date/#date-id96473 on how to use the LuaDate library.
+That's why qsv bundles & preloads LuaDate v2.2.1 as date manipulation is a common task.
+See https://tieske.github.io/date/ on how to use the LuaDate library.
 
 Additional libraries can be loaded from the LUAU_PATH using luau's "require" function.
 See https://github.com/LewisJEllis/awesome-lua for a list of other libraries.
