@@ -6,8 +6,8 @@ Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame exp
 (see https://docs.pola.rs/user-guide/sql/intro/).
 
 For a list of SQL functions and keywords supported by Polars SQL, see
-https://github.com/pola-rs/polars/blob/rs-0.39.2/crates/polars-sql/src/functions.rs
-https://github.com/pola-rs/polars/blob/rs-0.39.2/crates/polars-sql/src/keywords.rs and
+https://github.com/pola-rs/polars/blob/rs-0.40.0/crates/polars-sql/src/functions.rs
+https://github.com/pola-rs/polars/blob/rs-0.40.0/crates/polars-sql/src/keywords.rs and
 https://github.com/pola-rs/polars/issues/7227
 
 Returns the shape of the query result (number of rows, number of columns) to stderr.
@@ -180,6 +180,10 @@ sqlp options:
                               null values when READING CSV files (e.g. NULL, NONE, <empty string>).
                               Use "<empty string>" to consider an empty string a null value.
                               [default: <empty string>]
+    --decimal-comma           Use comma as the decimal separator when parsing CSVs.
+                              Otherwise, use period as the decimal separator.
+                              Note that you'll need to set --delimiter to an alternate delimiter
+                              other than the default comma if you are using this option.
 
                               CSV OUTPUT FORMAT ONLY:
     --datetime-format <fmt>   The datetime format to use writing datetimes.
@@ -262,6 +266,7 @@ struct Args {
     flag_no_optimizations:      bool,
     flag_ignore_errors:         bool,
     flag_truncate_ragged_lines: bool,
+    flag_decimal_comma:         bool,
     flag_datetime_format:       Option<String>,
     flag_date_format:           Option<String>,
     flag_time_format:           Option<String>,
@@ -554,18 +559,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
     } else {
         polars::lazy::frame::OptState {
-            projection_pushdown: true,
-            predicate_pushdown:  true,
-            type_coercion:       true,
-            simplify_expr:       true,
-            file_caching:        !args.flag_low_memory,
-            slice_pushdown:      true,
-            comm_subplan_elim:   !args.flag_low_memory,
-            comm_subexpr_elim:   true,
-            streaming:           args.flag_low_memory,
-            fast_projection:     true,
-            eager:               false,
-            row_estimate:        true,
+            projection_pushdown:  true,
+            predicate_pushdown:   true,
+            cluster_with_columns: true,
+            type_coercion:        true,
+            simplify_expr:        true,
+            file_caching:         !args.flag_low_memory,
+            slice_pushdown:       true,
+            comm_subplan_elim:    !args.flag_low_memory,
+            comm_subexpr_elim:    true,
+            streaming:            args.flag_low_memory,
+            fast_projection:      true,
+            eager:                false,
+            row_estimate:         true,
         }
     };
     // gated by log::log_enabled!(log::Level::Debug) to avoid the
@@ -617,6 +623,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         && !args.flag_truncate_ragged_lines
         && !args.flag_ignore_errors
         && args.flag_rnull_values.is_empty()
+        && !args.flag_decimal_comma
         && comment_char.is_none()
         && std::path::Path::new(&args.arg_input[0])
             .extension()
@@ -673,7 +680,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 );
             }
             let lf = LazyCsvReader::new(table)
-                .has_header(true)
+                .with_has_header(true)
                 .with_missing_is_null(true)
                 .with_comment_prefix(comment_char.as_deref())
                 .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
@@ -681,8 +688,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 .with_infer_schema_length(Some(args.flag_infer_len))
                 .with_try_parse_dates(args.flag_try_parsedates)
                 .with_ignore_errors(args.flag_ignore_errors)
-                .truncate_ragged_lines(args.flag_truncate_ragged_lines)
-                .low_memory(args.flag_low_memory)
+                .with_truncate_ragged_lines(args.flag_truncate_ragged_lines)
+                .with_decimal_comma(args.flag_decimal_comma)
+                .with_low_memory(args.flag_low_memory)
                 .finish()?;
             ctx.register(table_name, lf.with_optimizations(optimization_state));
         }
