@@ -2,13 +2,14 @@ static USAGE: &str = r#"
 Compute summary statistics & infers data types for each column in a CSV. 
 
 Summary statistics includes sum, min/max/range, min/max length, mean, stddev, variance,
-nullcount, sparsity, quartiles, interquartile range (IQR), lower/upper fences, skewness, median, 
-cardinality, mode/s & "antimode/s", and median absolute deviation (MAD). Note that some
-statistics requires loading the entire file into memory, so they must be enabled explicitly. 
+nullcount, max_precision, sparsity, quartiles, interquartile range (IQR), lower/upper fences,
+skewness, median, cardinality, mode/s & "antimode/s", and median absolute deviation (MAD).
+Note that some statistics require loading the entire file into memory, so they must be
+enabled explicitly. 
 
 By default, the following "streaming" statistics are reported for *every* column:
-sum, min/max/range values, min/max length, mean, stddev, variance, nullcount & sparsity.
-The default set of statistics corresponds to statistics that can be computed efficiently
+sum, min/max/range values, min/max length, mean, stddev, variance, nullcount, max_precision and
+sparsity. The default set of statistics corresponds to ones that can be computed efficiently
 on a stream of data (i.e., constant memory) and works with arbitrarily large CSV files.
 
 The following additional "non-streaming" statistics require loading the entire file into memory:
@@ -33,6 +34,7 @@ Date range, stddev, variance, MAD & IQR are returned in days, not timestamp mill
 
 Each column's data type is also inferred (NULL, Integer, String, Float, Date, DateTime and
 Boolean with --infer-boolean option).
+For String data types, it also determines if the column is all ASCII characters.
 Unlike the sniff command, stats' data type inferences are GUARANTEED, as the entire file
 is scanned, and not just sampled.
 
@@ -41,7 +43,7 @@ as its an expensive operation to match a date candidate against 19 possible date
 with each format, having several variants.
 
 The date formats recognized and its sub-variants along with examples can be found at 
-https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats.
+https://github.com/jqnatividad/qsv-dateparser?tab=readme-ov-file#accepted-date-formats.
 
 Computing statistics on a large file can be made MUCH faster if you create an index for it
 first with 'qsv index' to enable multithreading. With an index, the file is split into equal
@@ -73,8 +75,8 @@ stats options:
     -s, --select <arg>        Select a subset of columns to compute stats for.
                               See 'qsv select --help' for the format details.
                               This is provided here because piping 'qsv select'
-                              into 'qsv stats' will disable the use of indexing.
-    -E, --everything          Show all statistics available.
+                              into 'qsv stats' will prevent the use of indexing.
+    -E, --everything          Compute all statistics available.
     --typesonly               Infer data types only and do not compute statistics.
                               Note that if you want to infer dates, you'll still need to use
                               the --infer-dates and --dates-whitelist options.
@@ -82,20 +84,21 @@ stats options:
                               the --cardinality option. When a column's cardinality is 2,
                               and the 2 values' first characters are 0/1, t/f & y/n
                               case-insensitive, the data type is inferred as boolean.
-    --mode                    Show the mode/s & antimode/s. Multimodal-aware.
+    --mode                    Compute the mode/s & antimode/s. Multimodal-aware.
                               This requires loading all CSV data in memory.
-    --cardinality             Show the cardinality.
+    --cardinality             Compute the cardinality.
                               This requires loading all CSV data in memory.
-    --median                  Show the median.
+    --median                  Compute the median.
                               This requires loading all CSV data in memory.
-    --mad                     Shows the median absolute deviation (MAD).
+    --mad                     Compute the median absolute deviation (MAD).
                               This requires loading all CSV data in memory.
-    --quartiles               Show the quartiles, the IQR, the lower/upper inner/outer
+    --quartiles               Compute the quartiles, the IQR, the lower/upper inner/outer
                               fences and skewness.
                               This requires loading all CSV data in memory.
     --round <decimal_places>  Round statistics to <decimal_places>. Rounding is done following
                               Midpoint Nearest Even (aka "Bankers Rounding") rule.
                               https://docs.rs/rust_decimal/latest/rust_decimal/enum.RoundingStrategy.html
+                              If set to the sentinel value 9999, no rounding is done.
                               For dates - range, stddev & IQR are always at least 5 decimal places as
                               they are reported in days, and 5 places gives us millisecond precision.
                               [default: 4]
@@ -133,12 +136,23 @@ stats options:
                               number of CPUs detected.
     --stats-binout            Write the stats in binary format. This is used internally
                               by other qsv commands (currently `schema` & `tojsonl`) to
-                              load cached stats into memory faster. If set, the binary
-                              encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
+                              load cached stats into memory faster. If set, the snappy compressed
+                              binary encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
                               You can preemptively create the binary encoded stats file
-                              by using this option before running the `schema` and `tojsonl`
+                              by using this option BEFORE running the `schema` and `tojsonl`
                               commands and they will automatically load the binary encoded
                               stats file if it exists.
+ -c, --cache-threshold <arg>  When greater than 1, the threshold in milliseconds before caching
+                              stats results. If a stats run takes longer than this threshold,
+                              the stats results will be cached.
+                              Set to 0 to suppress caching. 
+                              Set to 1 to force caching.
+                              Set to a negative number to automatically create an index
+                              when the input file size is greater than abs(arg) in bytes.
+                              If the negative number ends with 5, it will delete the index
+                              file and the stats cache file after the stats run. Otherwise,
+                              the index file and the cache files are kept.
+                              [default: 5000]
 
 Common options:
     -h, --help             Display this message
@@ -169,7 +183,7 @@ It's type inferences are also used by the `tojsonl` command to generate properly
 JSONL files.
 
 To safeguard against undefined behavior, `stats` is the most extensively tested command,
-with >480 tests.
+with ~500 tests.
 */
 
 use std::{
@@ -219,6 +233,7 @@ pub struct Args {
     pub flag_force:           bool,
     pub flag_jobs:            Option<usize>,
     pub flag_stats_binout:    bool,
+    pub flag_cache_threshold: isize,
     pub flag_output:          Option<String>,
     pub flag_no_headers:      bool,
     pub flag_delimiter:       Option<Delimiter>,
@@ -266,6 +281,14 @@ const MS_IN_DAY_INT: i64 = 86_400_000;
 // 5 decimal places give us millisecond precision
 const DAY_DECIMAL_PLACES: u32 = 5;
 
+// maximum number of output columns
+const MAX_STAT_COLUMNS: usize = 32;
+
+// maximum number of antimodes to display
+const MAX_ANTIMODES: usize = 10;
+// maximum length of antimode string before truncating and appending "..."
+const MAX_ANTIMODE_LEN: usize = 100;
+
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
     if args.flag_typesonly {
@@ -368,6 +391,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut stats_for_encoding: Vec<Stats> = Vec::new();
 
     let mut compute_stats = true;
+    let mut create_cache = args.flag_cache_threshold > 0 || args.flag_stats_binout;
+    let mut autoindex_set = false;
 
     let write_stats_binout = args.flag_stats_binout;
 
@@ -471,6 +496,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 util::mem_file_check(&path, false, args.flag_memcheck)?;
             }
 
+            // check if flag_cache_threshold is a negative number,
+            // if so, set the autoindex_size to absolute of the number
+            if args.flag_cache_threshold.is_negative() {
+                fconfig.autoindex_size = args.flag_cache_threshold.unsigned_abs() as u64;
+                autoindex_set = true;
+            }
+
             // we need to count the number of records in the file to calculate sparsity
             let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&fconfig).unwrap());
             log::info!("scanning {record_count} records...");
@@ -511,14 +543,25 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
 
             // update the stats args json metadata
-            current_stats_args.canonical_input_path =
-                path.canonicalize()?.to_str().unwrap().to_string();
-            current_stats_args.record_count = *record_count;
             current_stats_args.compute_duration_ms = start_time.elapsed().as_millis() as u64;
-            current_stats_args.date_generated = chrono::Utc::now().to_rfc3339();
+
+            if create_cache
+                && current_stats_args.compute_duration_ms > args.flag_cache_threshold as u64
+            {
+                // if the stats run took longer than the cache threshold and the threshold > 0,
+                // cache the stats so we don't have to recompute it next time
+                current_stats_args.canonical_input_path =
+                    path.canonicalize()?.to_str().unwrap().to_string();
+                current_stats_args.record_count = *record_count;
+                current_stats_args.date_generated = chrono::Utc::now().to_rfc3339();
+            }
         }
     }
 
+    // ensure create_cache is also true if the user specified --cache-threshold 1
+    create_cache =
+        create_cache || args.flag_cache_threshold == 1 || args.flag_cache_threshold.is_negative();
+
     wtr.flush()?;
 
     if let Some(pb) = stdin_tempfile_path {
@@ -557,7 +600,32 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
         }
 
-        if compute_stats {
+        if args.flag_cache_threshold.is_negative() && args.flag_cache_threshold % 10 == -5 {
+            // if the cache threshold is a negative number ending in 5,
+            // delete both the index file and the stats cache file
+            if autoindex_set {
+                let index_file = path.with_extension("csv.idx");
+                log::debug!("deleting index file: {}", index_file.display());
+                if std::fs::remove_file(index_file.clone()).is_err() {
+                    // fails silently if it can't remove the index file
+                    log::warn!("Could not remove index file: {}", index_file.display());
+                }
+            }
+            create_cache = false;
+        }
+
+        if !create_cache {
+            // remove the stats cache file
+            if fs::remove_file(stats_pathbuf.clone()).is_err() {
+                // fails silently if it can't remove the stats file
+                log::warn!(
+                    "Could not remove stats cache file: {}",
+                    stats_pathbuf.display()
+                );
+            }
+        }
+
+        if compute_stats && create_cache {
             // save the stats args to "<FILESTEM>.stats.csv.json"
             // if we computed the stats
             stats_pathbuf.set_extension("csv.json");
@@ -650,11 +718,12 @@ impl Args {
         let pool = ThreadPool::new(util::njobs(self.flag_jobs));
         let (send, recv) = channel::bounded(0);
         for i in 0..nchunks {
-            // safety: the index file call is always safe seeking may fail and you have a bigger
-            // problem as the index file was modified WHILE stats is running and you
-            // NEED to abort if that happens, however unlikely
             let (send, args, sel) = (send.clone(), self.clone(), sel.clone());
             pool.execute(move || {
+                // safety: indexed() is safe as we know we have an index file
+                // if it does return an Err, you have a bigger problem as the index file was
+                // modified WHILE stats is running and you NEED to abort if that
+                // happens, however unlikely
                 let mut idx = args.rconfig().indexed().unwrap().unwrap();
                 idx.seek((i * chunk_size) as u64)
                     .expect("File seek failed.");
@@ -703,16 +772,18 @@ impl Args {
 
         // safety: we know INFER_DATE_FLAGS is Some because we called init_date_inference
         let infer_date_flags = INFER_DATE_FLAGS.get().unwrap();
+
         // so we don't need to get infer_boolean/prefer_dmy from big args struct for each iteration
+        // and hopefully the compiler will optimize this and use registers in the hot loop
         let infer_boolean = self.flag_infer_boolean;
         let prefer_dmy = self.flag_prefer_dmy;
 
-        // safety: because we're using iterators and INFER_DATE_FLAGS has the same size,
-        // we know we don't need to bounds check
-        unsafe {
-            let mut i;
-            for row in it {
-                i = 0;
+        let mut i;
+        for row in it {
+            i = 0;
+            // safety: because we're using iterators and INFER_DATE_FLAGS has the same size,
+            // we know we don't need to bounds check
+            unsafe {
                 for field in sel.select(&row.unwrap_unchecked()) {
                     stats.get_unchecked_mut(i).add(
                         field,
@@ -771,11 +842,12 @@ impl Args {
             return csv::StringRecord::from(vec!["field", "type"]);
         }
 
-        // with --everything, we have 30 columns at most
-        let mut fields = Vec::with_capacity(30);
+        // with --everything, we have 32 columns at most
+        let mut fields = Vec::with_capacity(MAX_STAT_COLUMNS);
         fields.extend_from_slice(&[
             "field",
             "type",
+            "is_ascii",
             "sum",
             "min",
             "max",
@@ -786,6 +858,7 @@ impl Args {
             "stddev",
             "variance",
             "nullcount",
+            "max_precision",
             "sparsity",
         ]);
         let all = self.flag_everything;
@@ -914,16 +987,18 @@ impl Commute for WhichStats {
 
 #[derive(Clone, Serialize, Deserialize, PartialEq)]
 pub struct Stats {
-    typ:       FieldType,
-    sum:       Option<TypedSum>,
-    minmax:    Option<TypedMinMax>,
-    online:    Option<OnlineStats>,
-    nullcount: u64,
-    modes:     Option<Unsorted<Vec<u8>>>,
-    median:    Option<Unsorted<f64>>,
-    mad:       Option<Unsorted<f64>>,
-    quartiles: Option<Unsorted<f64>>,
-    which:     WhichStats,
+    typ:           FieldType,
+    is_ascii:      bool,
+    sum:           Option<TypedSum>,
+    minmax:        Option<TypedMinMax>,
+    online:        Option<OnlineStats>,
+    nullcount:     u64,
+    max_precision: u16,
+    modes:         Option<Unsorted<Vec<u8>>>,
+    median:        Option<Unsorted<f64>>,
+    mad:           Option<Unsorted<f64>>,
+    quartiles:     Option<Unsorted<f64>>,
+    which:         WhichStats,
 }
 
 #[inline]
@@ -966,10 +1041,12 @@ impl Stats {
         }
         Stats {
             typ: FieldType::default(),
+            is_ascii: true,
             sum,
             minmax,
             online,
             nullcount: 0,
+            max_precision: 0,
             modes,
             median,
             mad,
@@ -1017,8 +1094,9 @@ impl Stats {
                     };
                 }
             },
-            // do nothing for String type
-            TString => {},
+            TString => {
+                self.is_ascii &= sample.is_ascii();
+            },
             TFloat | TInteger => {
                 if sample_type == TNull {
                     if self.which.include_nulls {
@@ -1040,6 +1118,20 @@ impl Stats {
                     if let Some(v) = self.online.as_mut() {
                         v.add(&n);
                     }
+                    if t == TFloat {
+                        let mut buffer = ryu::Buffer::new();
+                        // safety: we know that n is a valid f64
+                        // so there will always be a fraction part, even if it's 0
+                        let fractpart = buffer.format_finite(n).split('.').next_back().unwrap();
+                        self.max_precision = std::cmp::max(
+                            self.max_precision,
+                            (if *fractpart == *"0" {
+                                0
+                            } else {
+                                fractpart.len()
+                            }) as u16,
+                        );
+                    }
                 }
             },
             TDateTime | TDate => {
@@ -1082,8 +1174,8 @@ impl Stats {
 
         let typ = self.typ;
         // prealloc memory for performance
-        // we have 30 columns at most with --everything
-        let mut pieces = Vec::with_capacity(30);
+        // we have 32 columns at most with --everything
+        let mut pieces = Vec::with_capacity(MAX_STAT_COLUMNS);
 
         let empty = String::new;
 
@@ -1136,7 +1228,7 @@ impl Stats {
 
                         // We only store the first 10 antimodes
                         // so if antimodes_count > 10, add the "*PREVIEW: " prefix
-                        if antimodes_count > 10 {
+                        if antimodes_count > MAX_ANTIMODES {
                             antimodes_list.push_str("*PREVIEW: ");
                         }
 
@@ -1150,8 +1242,8 @@ impl Stats {
                         antimodes_list.push_str(antimodes_vals);
 
                         // and truncate at 100 characters with an ellipsis
-                        if antimodes_list.len() > 100 {
-                            util::utf8_truncate(&mut antimodes_list, 101);
+                        if antimodes_list.len() > MAX_ANTIMODE_LEN {
+                            util::utf8_truncate(&mut antimodes_list, MAX_ANTIMODE_LEN + 1);
                             antimodes_list.push_str("...");
                         }
 
@@ -1204,6 +1296,13 @@ impl Stats {
             return csv::StringRecord::from(pieces);
         }
 
+        // is_ascii
+        if typ == FieldType::TString {
+            pieces.push(self.is_ascii.to_string());
+        } else {
+            pieces.push(empty());
+        }
+
         // sum
         if let Some(sum) = self.sum.as_ref().and_then(|sum| sum.show(typ)) {
             if typ == FieldType::TFloat {
@@ -1266,10 +1365,17 @@ impl Stats {
         let mut buffer = itoa::Buffer::new();
         pieces.push(buffer.format(self.nullcount).to_owned());
 
+        // max precision
+        if typ == TFloat {
+            pieces.push(self.max_precision.to_string());
+        } else {
+            pieces.push(empty());
+        }
+
         // sparsity
         // stats is also called by the `schema` and `tojsonl` commands to infer a schema,
-        // sparsity is not required by those cmds and we don't necessarily have the
-        // record_count when called by those cmds, so just set sparsity to nullcount
+        // sparsity is not required by those cmds and we don't necessarily have a valid
+        // RECORD_COUNT when called by those cmds, so just set sparsity to nullcount
         // (i.e. divide by 1) so we don't panic.
         #[allow(clippy::cast_precision_loss)]
         let sparsity: f64 = self.nullcount as f64 / *RECORD_COUNT.get().unwrap_or(&1) as f64;
@@ -1339,7 +1445,7 @@ impl Stats {
                 let iqr = q3 - q1;
 
                 // use fused multiply add (mul_add) when possible
-                // fused mul_add is more accurate & may be more performant if the
+                // fused mul_add is more accurate & is more performant if the
                 // target architecture has a dedicated `fma` CPU instruction
                 // https://doc.rust-lang.org/std/primitive.f64.html#method.mul_add
 
@@ -1408,10 +1514,12 @@ impl Commute for Stats {
     #[inline]
     fn merge(&mut self, other: Stats) {
         self.typ.merge(other.typ);
+        self.is_ascii &= other.is_ascii;
         self.sum.merge(other.sum);
         self.minmax.merge(other.minmax);
         self.online.merge(other.online);
         self.nullcount += other.nullcount;
+        self.max_precision = std::cmp::max(self.max_precision, other.max_precision);
         self.modes.merge(other.modes);
         self.median.merge(other.median);
         self.quartiles.merge(other.quartiles);
@@ -1457,8 +1565,8 @@ impl FieldType {
             return (FieldType::TString, None);
         }
 
-        let utf8_string = if current_type == FieldType::TFloat
-            || current_type == FieldType::TInteger
+        let utf8_string = if current_type == FieldType::TInteger
+            || current_type == FieldType::TFloat
             || current_type == FieldType::TNull
         {
             if let Ok(int_val) = atoi_simd::parse::<i64>(sample) {
@@ -1556,7 +1664,6 @@ impl fmt::Display for FieldType {
     }
 }
 
-#[cfg(debug_assertions)]
 impl fmt::Debug for FieldType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
@@ -1683,7 +1790,7 @@ impl TypedMinMax {
                 self.floats.add(n as f64);
             },
             // it must be a TDate or TDateTime
-            // we use _ here for the match to avoid
+            // we use "_" here instead of "TDate | TDateTime" for the match to avoid
             // the overhead of matching on the OR value, however minor
             _ => {
                 let n = atoi_simd::parse::<i64>(sample).unwrap();
