@@ -1237,6 +1237,12 @@ impl ColumnNameParser {
 pub fn round_num(dec_f64: f64, places: u32) -> String {
     use rust_decimal::{Decimal, RoundingStrategy};
 
+    // if places is the sentinel value 9999, we don't round, just return the number as is
+    if places == 9999 {
+        let mut buffer = ryu::Buffer::new();
+        return buffer.format(dec_f64).to_owned();
+    }
+
     // use from_f64_retain, so we have all the excess bits before rounding with
     // round_dp_with_strategy as from_f64 will prematurely round when it drops the excess bits
     let Some(dec_num) = Decimal::from_f64_retain(dec_f64) else {
@@ -1666,3 +1672,269 @@ pub fn format_systemtime(time: SystemTime, format_specifier: &str) -> String {
     let datetime = chrono::DateTime::from_timestamp(timestamp as i64, 0).unwrap_or_default();
     format!("{datetime}", datetime = datetime.format(format_specifier))
 }
+
+pub fn create_json_writer(
+    output: &Option<String>,
+    buffer_capacity: usize,
+) -> std::io::Result<Box<dyn Write + Send + 'static>> {
+    // create a JSON writer
+    // if flag_output is None or "-" then write to stdout
+    let output = output.as_ref().map_or("-", |s| s.as_str());
+    let buffer_size = if buffer_capacity == 0 {
+        config::DEFAULT_WTR_BUFFER_CAPACITY
+    } else {
+        buffer_capacity
+    };
+    let writer: Box<dyn Write + Send + 'static> = match output {
+        "-" => Box::new(std::io::BufWriter::with_capacity(
+            buffer_size,
+            std::io::stdout(),
+        )),
+        "stderr" => Box::new(std::io::BufWriter::with_capacity(
+            buffer_size,
+            std::io::stderr(),
+        )),
+        _ => Box::new(std::io::BufWriter::with_capacity(
+            buffer_size,
+            fs::File::create(output)?,
+        )),
+    };
+    Ok(writer)
+}
+
+/// iterate over the CSV ByteRecords and write them to the JSON file
+pub fn write_json(
+    output: &Option<String>,
+    no_headers: bool,
+    headers: &csv::ByteRecord,
+    records: impl Iterator<Item = csv::ByteRecord>,
+) -> CliResult<()> {
+    let mut json_wtr = create_json_writer(output, config::DEFAULT_WTR_BUFFER_CAPACITY * 4)?;
+
+    let header_vec: Vec<String> = headers
+        .iter()
+        .enumerate()
+        .map(|(col_idx, b)| {
+            if no_headers {
+                col_idx.to_string()
+            } else {
+                String::from_utf8_lossy(b).to_string()
+            }
+        })
+        .collect();
+
+    // Write the opening bracket for the JSON array
+    write!(json_wtr, "[")?;
+    let mut is_first = true;
+
+    let rec_len = header_vec.len().saturating_sub(1);
+    let mut temp_val;
+    let null_val = "null".to_string();
+    let mut json_string_val: serde_json::Value;
+
+    for record in records {
+        if is_first {
+            is_first = false;
+        } else {
+            // Write a comma before each record except the first one
+            write!(json_wtr, ",")?;
+        }
+        write!(json_wtr, "{{")?;
+        for (idx, b) in record.iter().enumerate() {
+            if let Ok(val) = simdutf8::basic::from_utf8(b) {
+                temp_val = val.to_owned();
+            } else {
+                temp_val = String::from_utf8_lossy(b).to_string();
+            }
+            if temp_val.is_empty() {
+                temp_val.clone_from(&null_val);
+            } else {
+                // we round-trip the value to serde_json
+                // to escape the string properly per JSON spec
+                json_string_val = serde_json::Value::String(temp_val);
+                temp_val = json_string_val.to_string();
+            }
+            // safety: idx is always in bounds
+            // so we can get_unchecked here
+            if idx < rec_len {
+                unsafe {
+                    write!(
+                        &mut json_wtr,
+                        r#""{key}":{value},"#,
+                        key = header_vec.get_unchecked(idx),
+                        value = temp_val
+                    )?;
+                }
+            } else {
+                // last column in the JSON record, no comma
+                unsafe {
+                    write!(
+                        &mut json_wtr,
+                        r#""{key}":{value}"#,
+                        key = header_vec.get_unchecked(idx),
+                        value = temp_val
+                    )?;
+                }
+            }
+        }
+        write!(json_wtr, "}}")?;
+    }
+    // Write the closing bracket for the JSON array
+    writeln!(json_wtr, "]")?;
+
+    Ok(json_wtr.flush()?)
+}
+
+/// write a single csv::ByteRecord to a JSON record writer
+/// if no_headers is true, the column index (0-based) is used as the key
+/// if no_headers is false, the header is used as the key
+/// if is_first is true, a comma is not written before the record
+/// if is_first is false, a comma is written before the record
+/// is_first is passed as a mutable reference so that it can be updated
+/// in this helper function efficiently
+/// in this way, we can stream JSON records to a writer
+pub fn write_json_record<W: std::io::Write>(
+    json_wtr: &mut W,
+    no_headers: bool,
+    headers: &csv::ByteRecord,
+    record: &csv::ByteRecord,
+    is_first: &mut bool,
+) -> std::io::Result<()> {
+    let header_vec: Vec<String> = headers
+        .iter()
+        .enumerate()
+        .map(|(col_idx, b)| {
+            if no_headers {
+                col_idx.to_string()
+            } else {
+                String::from_utf8_lossy(b).to_string()
+            }
+        })
+        .collect();
+
+    let rec_len = header_vec.len().saturating_sub(1);
+    let mut temp_val;
+    let mut json_string_val: serde_json::Value;
+    let null_val = "null".to_string();
+
+    if *is_first {
+        write!(json_wtr, "{{")?;
+        *is_first = false;
+    } else {
+        write!(json_wtr, ",{{")?;
+    }
+    for (idx, b) in record.iter().enumerate() {
+        if let Ok(val) = simdutf8::basic::from_utf8(b) {
+            temp_val = val.to_owned();
+        } else {
+            temp_val = String::from_utf8_lossy(b).to_string();
+        }
+        if temp_val.is_empty() {
+            temp_val.clone_from(&null_val);
+        } else {
+            json_string_val = serde_json::Value::String(temp_val);
+            temp_val = json_string_val.to_string();
+        }
+        if idx < rec_len {
+            unsafe {
+                write!(
+                    json_wtr,
+                    r#""{key}":{value},"#,
+                    key = header_vec.get_unchecked(idx),
+                    value = temp_val
+                )?;
+            }
+        } else {
+            unsafe {
+                write!(
+                    json_wtr,
+                    r#""{key}":{value}"#,
+                    key = header_vec.get_unchecked(idx),
+                    value = temp_val
+                )?;
+            }
+        }
+    }
+    Ok(write!(json_wtr, "}}")?)
+}
+
+// comment out for now as this is still WIP
+// pub fn create_json_record(
+//     no_headers: bool,
+//     headers: &csv::ByteRecord,
+//     record: &csv::ByteRecord,
+//     is_first: &mut bool,
+// ) -> CliResult<String> {
+//     let header_vec: Vec<String> = headers
+//         .iter()
+//         .enumerate()
+//         .map(|(col_idx, b)| {
+//             if no_headers {
+//                 col_idx.to_string()
+//             } else {
+//                 String::from_utf8_lossy(b).to_string()
+//             }
+//         })
+//         .collect();
+
+//     let mut json_record = String::new();
+
+//     let rec_len = header_vec.len().saturating_sub(1);
+//     let mut temp_val;
+//     let mut json_string_val: serde_json::Value;
+//     let null_val = "null".to_string();
+
+//     if *is_first {
+//         // write!(json_wtr, "{{")?;
+//         json_record.push('{');
+//         *is_first = false;
+//     } else {
+//         // write!(json_wtr, ",{{")?;
+//         json_record.push_str(",{");
+//     }
+//     for (idx, b) in record.iter().enumerate() {
+//         if let Ok(val) = simdutf8::basic::from_utf8(b) {
+//             temp_val = val.to_owned();
+//         } else {
+//             temp_val = String::from_utf8_lossy(b).to_string();
+//         }
+//         if temp_val.is_empty() {
+//             temp_val.clone_from(&null_val);
+//         } else {
+//             json_string_val = serde_json::Value::String(temp_val);
+//             temp_val = json_string_val.to_string();
+//         }
+//         if idx < rec_len {
+//             unsafe {
+//                 // write!(
+//                 //     json_wtr,
+//                 //     r#""{key}":{value},"#,
+//                 //     key = header_vec.get_unchecked(idx),
+//                 //     value = temp_val
+//                 // )?;
+//                 json_record.push_str(&format!(
+//                     r#""{key}":{value},"#,
+//                     key = header_vec.get_unchecked(idx),
+//                     value = temp_val
+//                 ));
+//             }
+//         } else {
+//             unsafe {
+//                 // write!(
+//                 //     json_wtr,
+//                 //     r#""{key}":{value}"#,
+//                 //     key = header_vec.get_unchecked(idx),
+//                 //     value = temp_val
+//                 // )?;
+//                 json_record.push_str(&format!(
+//                     r#""{key}":{value}"#,
+//                     key = header_vec.get_unchecked(idx),
+//                     value = temp_val
+//                 ));
+//             }
+//         }
+//     }
+//     // Ok(write!(json_wtr, "}}")?)
+//     json_record.push('}');
+//     Ok(json_record)
+// }
