@@ -288,6 +288,43 @@ fn frequency_custom_other_text() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn frequency_custom_other_text_sorted() {
+    let (wrk, mut cmd) = setup("frequency_custom_other_text_sorted");
+    cmd.args(["--limit", "-4"])
+        .args(["--lmt-threshold", "4"])
+        .args(["--other-text", "Ibang halaga"])
+        .arg("--other-sorted");
+
+    let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    got.sort();
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["h1", "Ibang halaga (3)", "3", "42.85714"],
+        svec!["h1", "a", "4", "57.14286"],
+        svec!["h2", "Ibang halaga (4)", "7", "100"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn frequency_other_sorted() {
+    let (wrk, mut cmd) = setup("frequency_other_sorted");
+    cmd.args(["--limit", "-4"])
+        .args(["--lmt-threshold", "4"])
+        .arg("--other-sorted");
+
+    let mut got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    got.sort();
+    let expected = vec![
+        svec!["field", "value", "count", "percentage"],
+        svec!["h1", "Other (3)", "3", "42.85714"],
+        svec!["h1", "a", "4", "57.14286"],
+        svec!["h2", "Other (4)", "7", "100"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn frequency_other_text_none() {
     let (wrk, mut cmd) = setup("frequency_other_text_none");
