@@ -408,7 +408,7 @@ joinp_test!(
 );
 
 joinp_test!(joinp_full, |wrk: Workdir, mut cmd: process::Command| {
-    cmd.arg("--full");
+    cmd.arg("--full").arg("--coalesce");
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected1 = make_rows(
         false,
@@ -435,10 +435,48 @@ joinp_test!(joinp_full, |wrk: Workdir, mut cmd: process::Command| {
     assert!(got == expected1 || got == expected2);
 });
 
+joinp_test!(
+    joinp_full_not_coalesced,
+    |wrk: Workdir, mut cmd: process::Command| {
+        cmd.arg("--full");
+        let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+        let mut expected1 = make_rows(
+            false,
+            vec![
+                svec!["city", "state", "city_right", "place"],
+                svec!["Boston", "MA", "Boston", "Logan Airport"],
+                svec!["Boston", "MA", "Boston", "Boston Garden"],
+                svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
+                svec!["", "", "Orlando", "Disney World"],
+                svec!["San Francisco", "CA", "", ""],
+                svec!["New York", "NY", "", ""],
+            ],
+        );
+        // remove the first old header from expected1
+        expected1.remove(0);
+
+        let mut expected2 = make_rows(
+            false,
+            vec![
+                svec!["city", "state", "city_right", "place"],
+                svec!["Boston", "MA", "Boston", "Logan Airport"],
+                svec!["Boston", "MA", "Boston", "Boston Garden"],
+                svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
+                svec!["", "", "Orlando", "Disney World"],
+                svec!["New York", "NY", "", ""],
+                svec!["San Francisco", "CA", "", ""],
+            ],
+        );
+        expected2.remove(0);
+
+        assert!(got == expected1 || got == expected2);
+    }
+);
+
 joinp_test_compressed!(
     joinp_full_compressed,
     |wrk: Workdir, mut cmd: process::Command| {
-        cmd.arg("--full");
+        cmd.arg("--full").arg("--coalesce");
         let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
         let expected1 = make_rows(
             false,
@@ -469,7 +507,7 @@ joinp_test_compressed!(
 joinp_test_comments!(
     joinp_full_comments,
     |wrk: Workdir, mut cmd: process::Command| {
-        cmd.arg("--full");
+        cmd.arg("--full").arg("--coalesce");
         let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
         let expected1 = make_rows(
             false,
