@@ -144,6 +144,7 @@ fn sqlp_join_cross() {
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_groupby_orderby() {
     let wrk = Workdir::new("sqlp_boston311_groupby_orderby");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -205,6 +206,7 @@ fn sqlp_boston311_groupby_orderby() {
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_groupby_orderby_with_table_alias() {
     let wrk = Workdir::new("sqlp_boston311_groupby_orderby");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -294,6 +296,35 @@ fn sqlp_boston311_wnull_value() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_decimal_comma_issue_1050() {
+    let wrk = Workdir::new("sqlp_decimal_comma_issue_1050");
+    let test_file = wrk.load_test_file("progetti_sample_10.csv");
+
+    let mut cmd = wrk.command("sqlp");
+
+    cmd.arg(&test_file)
+        .arg("--decimal-comma")
+        .args(["--delimiter", ";"])
+        .arg("select COD_LOCALE_PROGETTO, FINANZ_UE from _t_1");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["COD_LOCALE_PROGETTO;FINANZ_UE"],
+        svec!["1AGCOE1627;6328008.44"],
+        svec!["1AGCOE2113;344203.2"],
+        svec!["1AGCOE645;491260.0"],
+        svec!["1AGCOE705;522811.32"],
+        svec!["1AGCOE706;460463.85"],
+        svec!["1ANPALINP-001;141566507.74"],
+        svec!["1ANPALINP-CLP-00002;47325000.0"],
+        svec!["1ANPALINP-CLP-00003;78450000.0"],
+        svec!["1ANPALINP-CLP-00004;67500000.0"],
+        svec!["1ANPALVAO1C001;6416.62"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_null_aware_equality_checks() {
     let wrk = Workdir::new("sqlp_null_aware_equality_checks");
@@ -632,18 +663,18 @@ fn sqlp_boston311_try_parsedates() {
         svec!["ward", "avg_tat"],
         svec!["Ward 11", "4847760000000.0"],
         svec!["01", "4818270000000.0"],
-        svec!["Ward 13", "1518365700000.0"],
+        svec!["Ward 13", "1518365750000.0"],
         svec!["Ward 15", "1278926000000.0"],
         svec!["Ward 21", "878446000000.0"],
         svec!["Ward 14", "618933000000.0"],
-        svec!["Ward 3", "437691450000.0"],
+        svec!["Ward 3", "437691444444.0"],
         svec!["Ward 5", "411909500000.0"],
         svec!["Ward 20", "367233000000.0"],
         svec!["9", "353495000000.0"],
         svec!["Ward 18", "249882000000.0"],
         svec!["19", "212566000000.0"],
         svec!["Ward 4", "112872600000.0"],
-        svec!["Ward 1", "107850670000.0"],
+        svec!["Ward 1", "107850666666.0"],
         svec!["Ward 10", "104110000000.0"],
         svec!["16", "93557000000.0"],
         svec!["Ward 19", "84164000000.0"],
@@ -654,8 +685,8 @@ fn sqlp_boston311_try_parsedates() {
         svec!["3", "68836600000.0"],
         svec!["Ward 9", "64097000000.0"],
         svec!["Ward 12", "62930000000.0"],
-        svec!["Ward 6", "54770168000.0"],
-        svec!["Ward 7", "38346334000.0"],
+        svec!["Ward 6", "54770166666.0"],
+        svec!["Ward 7", "38346333333.0"],
         svec!["Ward 8", "32767500000.0"],
         svec!["03", "29810500000.0"],
         svec!["07", "25328000000.0"],
@@ -664,7 +695,7 @@ fn sqlp_boston311_try_parsedates() {
         svec!["Ward 22", "13524000000.0"],
         svec!["1", "9469000000.0"],
         svec!["06", "5290000000.0"],
-        svec!["Ward 16", "4533667000.0"],
+        svec!["Ward 16", "4533666666.0"],
         svec!["8", "1757000000.0"],
         svec!["02", "1650000000.0"],
         svec!["18", "507000000.0"],
@@ -690,11 +721,11 @@ fn sqlp_boston311_try_parsedates_precision() {
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["ward", "avg_tat"],
-        svec!["Ward 11", "4847759785984.000"],
-        svec!["01", "4818270158848.000"],
-        svec!["Ward 13", "1518365704192.000"],
-        svec!["Ward 15", "1278925996032.000"],
-        svec!["Ward 21", "878445985792.000"],
+        svec!["Ward 11", "4847760000000.000"],
+        svec!["01", "4818270000000.000"],
+        svec!["Ward 13", "1518365750000.000"],
+        svec!["Ward 15", "1278926000000.000"],
+        svec!["Ward 21", "878446000000.000"],
     ];
     assert_eq!(got, expected);
 }
@@ -809,7 +840,7 @@ fn sqlp_boston311_explain() {
     let expected_begin = r#"Logical Plan
 "SORT BY [col(""avg_tat""), col(""ward"")]"
   AGGREGATE
-"  	[[(col(""closed_dt"")) - (col(""open_dt""))].mean().cast(Float32).alias(""avg_tat"")] BY [col(""ward"")] FROM"
+"  	[[(col(""closed_dt"")) - (col(""open_dt""))].mean().strict_cast(Float64).alias(""avg_tat"")] BY [col(""ward"")] FROM"
 ""
       Csv SCAN"#;
     assert!(got.starts_with(expected_begin));
@@ -821,6 +852,7 @@ fn sqlp_boston311_explain() {
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_sql_script() {
     let wrk = Workdir::new("sqlp_boston311_sql_script");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -857,6 +889,7 @@ select ward,count(*) as cnt from temp_table2 group by ward order by cnt desc, wa
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_sql_script_json() {
     let wrk = Workdir::new("sqlp_boston311_sql_script_json");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -880,6 +913,7 @@ select ward,count(*) as cnt from temp_table2 group by ward order by cnt desc, wa
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_sql_script_jsonl() {
     let wrk = Workdir::new("sqlp_boston311_sql_script_jsonl");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -911,6 +945,7 @@ select ward,count(*) as cnt from temp_table2 group by ward order by cnt desc, wa
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_cte_script() {
     let wrk = Workdir::new("sqlp_boston311_cte");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -938,6 +973,7 @@ select ward,count(*) as cnt from boston311_roxbury group by ward order by cnt de
 }
 
 #[test]
+// #[ignore = "temporarily disable due to a bug in polars aliasing"]
 fn sqlp_boston311_cte() {
     let wrk = Workdir::new("sqlp_boston311_cte");
     let test_file = wrk.load_test_file("boston311-100.csv");
@@ -1655,7 +1691,7 @@ fn sqlp_modulo() {
             svec!["1.5", "6", "11", "16.5"],
             svec!["", "7", "12", "17.0"],
             svec!["3.0", "8", "13", "18.5"],
-            svec!["4,333333333", "9", "14", ""],
+            svec!["4333333333", "9", "14", ""],
             svec!["5.0", "10", "15", "20.0"],
         ],
     );
@@ -1680,7 +1716,7 @@ fn sqlp_modulo() {
         svec!["1.5", "0.0", "3.0", "0.0"],
         svec!["", "1.0", "0.0", "0.5"],
         svec!["1.0", "2.0", "1.0", "2.0"],
-        svec!["", "0.0", "2.0", ""],
+        svec!["1.0", "0.0", "2.0", ""],
         svec!["1.0", "1.0", "3.0", "3.5"],
     ];
 
