@@ -622,7 +622,7 @@ fn stats_prefer_dmy() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -647,7 +647,7 @@ fn stats_prefer_mdy() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
 
@@ -672,7 +672,7 @@ fn stats_rounding() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-8places-stats.csv");
@@ -680,6 +680,27 @@ fn stats_rounding() {
     assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
 }
 
+#[test]
+fn stats_no_rounding() {
+    let wrk = Workdir::new("stats_no_rounding");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--everything")
+        .args(["--round", "9999"])
+        .arg(test_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    wrk.create("in2.csv", got);
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-everything-norounding-stats.csv");
+
+    // this should NOT BE EQUAL as floats are not rounded, and comparing floats is not reliable
+    assert_ne!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+}
+
 #[test]
 fn stats_no_date_inference() {
     let wrk = Workdir::new("stats_no_date_inference");
@@ -694,7 +715,7 @@ fn stats_no_date_inference() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-nodate-stats.csv");
@@ -720,7 +741,7 @@ fn stats_with_date_inference() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-date-stats.csv");
@@ -792,7 +813,7 @@ fn stats_with_date_type() {
 
     // removed variance & stddev columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("1-9,12-").arg("in2.csv");
+    cmd.arg("1-10,13-").arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-datenotime-stats.csv");
@@ -834,6 +855,203 @@ fn stats_typesonly_with_dates() {
     assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
 }
 
+#[test]
+fn stats_typesonly_cache_threshold_zero() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_typesonly_cache_threshold_zero");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--typesonly")
+        .arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        .args(["--cache-threshold", "0"])
+        .arg(test_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected = wrk.load_test_resource("boston311-100-typesonly-withdates-stats.csv");
+
+    assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
+
+    // check that the stats cache files were NOT created
+    assert!(!Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(!Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
+#[test]
+fn stats_typesonly_cache() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_typesonly_cache");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--typesonly")
+        .arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        .arg(test_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected = wrk.load_test_resource("boston311-100-typesonly-withdates-stats.csv");
+
+    assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
+
+    // check that the stats cache files were created
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
+#[test]
+fn stats_cache() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_cache");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        // set cache threshold to 1 to force cache creation
+        .args(["--cache-threshold", "1"])
+        .arg(test_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    wrk.create("in2.csv", got);
+
+    // removed variance & stddev columns as its causing flaky CI test for float values
+    let mut cmd = wrk.command("select");
+    cmd.arg("1-10,13-").arg("in2.csv");
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
+
+    assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+
+    // check that the stats cache files were created
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
+#[test]
+fn stats_cache_negative_threshold() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_cache_negative_threshold");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        // set cache threshold to -10240 to set autoindex_size to 10 kb
+        // and to force cache creation
+        .args(["-c", "-10240"])
+        .arg(test_file.clone());
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    // the index file SHOULD have been created as the input file size > 10 kb
+    assert!(Path::new(&format!("{test_file}.idx")).exists());
+
+    wrk.create("in2.csv", got);
+
+    // removed variance & stddev columns as its causing flaky CI test for float values
+    let mut cmd = wrk.command("select");
+    cmd.arg("1-10,13-").arg("in2.csv");
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
+
+    assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+
+    // check that the stats cache files were created
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
+#[test]
+fn stats_cache_negative_threshold_unmet() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_cache_negative_threshold_unmet");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        // set cache threshold to -51200 to set autoindex_size to 50 kb
+        // and to force cache creation
+        .args(["--cache-threshold", "-51200"])
+        .arg(test_file.clone());
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    // the index file SHOULD NOT have been created as the input file < 50 kb
+    assert!(!Path::new(&format!("{test_file}.idx")).exists());
+
+    wrk.create("in2.csv", got);
+
+    // removed variance & stddev columns as its causing flaky CI test for float values
+    let mut cmd = wrk.command("select");
+    cmd.arg("1-10,13-").arg("in2.csv");
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
+
+    assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+
+    // check that the stats cache files were created
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
+#[test]
+fn stats_cache_negative_threshold_five() {
+    use std::path::Path;
+
+    let wrk = Workdir::new("stats_cache_negative_threshold_five");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--infer-dates")
+        .arg("--dates-whitelist")
+        .arg("all")
+        // set cache threshold to -10245 to set autoindex_size to 10 kb
+        // this creates an index file, and then autodeletes it AND the stats cache files
+        .args(["-c", "-10245"])
+        .arg(test_file.clone());
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    // the index file WAS CREATED as the input file is > 10k
+    // but the index file WAS DELETED after stats exits as the threshold was negative
+    // and ends with a 5
+    assert!(!Path::new(&format!("{test_file}.idx")).exists());
+
+    wrk.create("in2.csv", got);
+
+    // removed variance & stddev columns as its causing flaky CI test for float values
+    let mut cmd = wrk.command("select");
+    cmd.arg("1-10,13-").arg("in2.csv");
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
+
+    assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+
+    // check that the stats cache files were created
+    assert!(!Path::new(&wrk.path("boston311-100.stats.csv")).exists());
+    assert!(!Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
+}
+
 #[test]
 fn stats_infer_boolean_1_0() {
     let wrk = Workdir::new("stats_infer_boolean_1_0");
@@ -879,6 +1097,28 @@ fn stats_typesonly_infer_boolean_t_f() {
     assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
 }
 
+#[test]
+fn stats_is_ascii() {
+    let wrk = Workdir::new("stats_is_ascii");
+    let test_file = wrk.load_test_file("boston311-100-with-nonascii.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg(test_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    wrk.create("in2.csv", got);
+
+    // removed variance & stddev columns as its causing flaky CI test for float values
+    let mut cmd = wrk.command("select");
+    cmd.arg("1-10,13-").arg("in2.csv");
+
+    let got2: String = wrk.stdout(&mut cmd);
+    let expected2 = wrk.load_test_resource("boston311-100-with-nonascii-stats.csv");
+
+    assert_eq!(dos2unix(&got2), dos2unix(&expected2).trim_end());
+}
+
 #[test]
 fn stats_everything_utf8_japanese_issue817() {
     let wrk = Workdir::new("stats_everything_utf8_japanese");
